@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
 # Please refer to the git history regarding who changed what and when in this file.
 
@@ -22,19 +23,6 @@ Base classes:
   MultidropIMAPRetrieverBase
 '''
 
-__all__ = [
-    'IMAPinitMixIn',
-    'IMAPRetrieverBase',
-    'IMAPSSLinitMixIn',
-    'MultidropPOP3RetrieverBase',
-    'MultidropIMAPRetrieverBase',
-    'POP3_SSL_PORT',
-    'POP3initMixIn',
-    'POP3RetrieverBase',
-    'POP3SSLinitMixIn',
-    'RetrieverSkeleton',
-]
-
 import sys
 import os
 import socket
@@ -69,6 +57,19 @@ except:
   SSLError = Exception
 import hashlib
 
+__all__ = [
+    'IMAPinitMixIn',
+    'IMAPRetrieverBase',
+    'IMAPSSLinitMixIn',
+    'MultidropPOP3RetrieverBase',
+    'MultidropIMAPRetrieverBase',
+    'POP3_SSL_PORT',
+    'POP3initMixIn',
+    'POP3RetrieverBase',
+    'POP3SSLinitMixIn',
+    'RetrieverSkeleton',
+]
+
 tocode = lambda x: isinstance(x,bytes) and x or x.encode()
 
 # If we have an ssl module:
@@ -1356,7 +1357,6 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         # so we do it explicitly here if we've deleted any messages.
         if self.deleted:
             self.conn.expunge()
-        self.conn.close()
         self.write_oldmailfile(self.mailbox_selected)
         # And clear some state
         self.mailbox_selected = False
@@ -1370,6 +1370,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         self.msgsizes = {}
         self.oldmail = {}
         self.__delivered = {}
+        self.conn.close()
 
     def select_mailbox(self, mailbox):
         self.log.trace()
@@ -1425,7 +1426,17 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         self.mailbox = mailbox
         self.uidvalidity = uidvalidity
 
-        self._getmsglist(count)
+        imap_search = self.conf['imap_search']
+        if imap_search:
+            result, data = self.conn.search(None, imap_search)
+            data = [d for d in data if d is not None]
+            if result == 'OK' and data:
+                msgs = b','.join(data[0].split())
+                if sys.version_info.major > 2:
+                    msgs = msgs.decode()
+                self._getmsglist(msgs)
+        else:
+            self._getmsglist(count)
 
         return count
 
@@ -1435,7 +1446,10 @@ class IMAPRetrieverBase(RetrieverSkeleton):
             if msgcount:
                 # Get UIDs and sizes for all messages in mailbox
                 response = self._parse_imapcmdresponse(
-                    'FETCH', '1:%d' % msgcount, '(UID RFC822.SIZE)'
+                    'FETCH',
+                    isinstance(msgcount,str) and msgcount or '1:%d'%msgcount,
+                    ('(UID)' if self.app_options['skip_imap_fetch_size'] else
+                    '(UID RFC822.SIZE)')
                 )
                 for line in response:
                     if not line:
@@ -1451,7 +1465,9 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                     self._mboxuids[msgid] = r['uid']
                     self._mboxuidorder.append(msgid)
                     self.msgnum_by_msgid[msgid] = None
-                    self.msgsizes[msgid] = int(r['rfc822.size'])
+                    self.msgsizes[msgid] = (0 if 
+                        self.app_options['skip_imap_fetch_size']
+                                        else int(r['rfc822.size']))
 
             # Remove messages from state file that are no longer in mailbox,
             # but only if the timestamp for them are old (30 days for now).
@@ -1486,7 +1502,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                 )
             self.log.debug('deleting message "%s"' % uid + os.linesep)
             response = self._parse_imapuidcmdresponse(
-                'STORE', uid, 'FLAGS', r'(\Deleted \Seen)'
+                'STORE', uid, 'FLAGS',
+                self.conf.get('imap_on_delete',None) or r'(\Deleted \Seen)'
             )
         except imaplib.IMAP4.error as o:
             raise getmailOperationError('IMAP error (%s)' % o)
@@ -1572,8 +1589,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
             return {}
 
         ext = re.search(
-            rb'X-GM-THRID (?P<THRID>\d+) X-GM-MSGID (?P<MSGID>\d+)'
-            rb' X-GM-LABELS \((?P<LABELS>.*)\) UID',
+            b'X-GM-THRID (?P<THRID>\\d+) X-GM-MSGID (?P<MSGID>\\d+)'
+            b' X-GM-LABELS \\((?P<LABELS>.*)\\) UID',
             response[0]
         )
         if not ext:
@@ -1759,9 +1776,12 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         except imaplib.IMAP4.error as o:
             return False
         except BrokenPipeError as o:
-            # The underlying TLS connection closed during IDLE
+            # The underlying connection closed during IDLE
             self.log.info('BrokenPipeError after IDLE\n')
             return False
+        except TimeoutError as o:
+            self.log.info('TimeoutError after IDLE\n')
+            return False
 
         if aborted:
             raise aborted
