@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
 # Please refer to the git history regarding who changed what and when in this file.
 
@@ -14,21 +15,10 @@ Currently implemented:
     recipient address patterns)
 '''
 
-__all__ = [
-    'DeliverySkeleton',
-    'Maildir',
-    'Mboxrd',
-    'MDA_qmaillocal',
-    'MDA_external',
-    'MultiDestinationBase',
-    'MultiDestination',
-    'MultiSorterBase',
-    'MultiSorter',
-]
-
 import sys
 import os
 import re
+import smtplib
 import types
 
 try:
@@ -42,6 +32,19 @@ from getmailcore.exceptions import *
 from getmailcore.utilities import *
 from getmailcore.baseclasses import *
 
+__all__ = [
+    'DeliverySkeleton',
+    'Maildir',
+    'Mboxrd',
+    'MDA_qmaillocal',
+    'MDA_external',
+    'MDA_lmtp',
+    'MultiDestinationBase',
+    'MultiDestination',
+    'MultiSorterBase',
+    'MultiSorter',
+]
+
 def exit_with(stderr,errmsg):
     stderr.write(errmsg.encode())
     stderr.flush()
@@ -272,7 +275,10 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
             # Open mbox file, refusing to create it if it doesn't exist
             fd = os.open(self.conf['path'], os.O_RDWR)
             status_old = os.fstat(fd)
-            f = os.fdopen(fd, 'br+')
+            try:
+                f = os.fdopen(fd, 'br+')
+            except ValueError: # Py2
+                f = os.fdopen(fd, 'r+')
             lock_file(f, self.conf['locktype'])
             # Check if it _is_ an mbox file.  mbox files must start with "From "
             # in their first line, or are 0-length files.
@@ -667,6 +673,85 @@ class MDA_external(DeliverySkeleton, ForkingBase):
             self.log.info('command %s: %s' % (self, child.err))
         return 'MDA_external command %s (%s)' % (self.conf['command'], child.out)
 
+
+class MDA_lmtp(DeliverySkeleton):
+    """LMTP server destination."""
+    _confitems = (
+        ConfInstance(name='configparser', required=False),
+        ConfString(name='host', required=True),
+        ConfInt(name='port', required=False, default=smtplib.LMTP_PORT),
+        ConfString(name='fallback', required=False, default=None),
+    )
+
+    def initialize(self):
+        if sys.version_info.major < 3:
+            raise getmailConfigurationError(
+                'MDA_lmtp in config: that was implemented only for python 3'
+            )
+        self.log.trace()
+        self.__connect()
+
+    def __connect(self):
+        try:
+            self.server = smtplib.LMTP(self.conf['host'], self.conf['port'])
+        except smtplib.SMTPException as err:
+            raise getmailConfigurationError(
+                'Failed to connect to server'
+            )
+
+    def __str__(self):
+        self.log.trace()
+        host = self.conf['host']
+        port = self.conf['port']
+        if host.startswith('/') or not port:
+            return 'MDA_lmtp at %s' % host
+        else:
+            return 'MDA_lmtp at %s:%d' % (host, port)
+
+    def showconf(self):
+        self.log.info('%s(%s)' % (type(self).__name__, self._confstring()))
+
+    def __send(self, msg, sender, recipient, __retrying=False):
+        try:
+            rcpt = self.server.send_message(msg, sender, recipient)
+        except smtplib.SMTPServerDisconnected as err:
+            if __retrying:
+                raise
+            else:
+                self.log.info('Lost connection to LMTP server, reconnecting')
+                self.__connect()
+                self.__send(msg, sender, recipient, __retrying=True)
+        except smtplib.SMTPRecipientsRefused as err:
+            rcpt = err.recipients
+        except smtplib.SMTPException as err:
+            raise getmailDeliveryError(
+                'LMTP error: %s: %s' % (type(err).__name__, err)
+            )
+
+        return rcpt
+
+    def _deliver_message(self, msg, delivered_to, received):
+        self.log.trace()
+        rcpt = self.__send(msg.content(), msg.sender, msg.recipient)
+
+        if msg.recipient in rcpt:
+            status, error = rcpt[msg.recipient]
+            fb_recipient = self.conf['fallback']
+            if 500 <= status <= 599 and fb_recipient:
+                fb_rcpt = self.__send(msg.content(), msg.sender, fb_recipient)
+                if fb_recipient in fb_rcpt:
+                    raise getmailDeliveryError(
+                        'Cannot deliver to intended or fallback target: %d %s'
+                        % fb_rcpt[fb_recipient]
+                    )
+            else:
+                raise getmailDeliveryError(
+                    'Cannot deliver to intended target: %d %s'
+                    % rcpt[msg.recipient]
+                )
+        return str(self)
+
+
 #######################################
 class MultiDestinationBase(DeliverySkeleton):
     '''Base class for destinations which hand messages off to other
