@@ -1,9 +1,26 @@
+# -*- coding: utf-8 -*-
 # docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
 # Please refer to the git history regarding who changed what and when in this file.
 
 '''Utility classes and functions for getmail.
 '''
 
+
+import os
+import os.path
+import socket
+import signal
+import stat
+import time
+import glob
+import re
+import fcntl
+import pwd
+import grp
+import getpass
+import subprocess
+import sys
+
 __all__ = [
     'address_no_brackets',
     'change_usergroup',
@@ -32,22 +49,6 @@ __all__ = [
     'unicode',
 ]
 
-
-import os
-import os.path
-import socket
-import signal
-import stat
-import time
-import glob
-import re
-import fcntl
-import pwd
-import grp
-import getpass
-import subprocess
-import sys
-
 if sys.version_info.major > 2:
     unicode = str
     tostr = lambda lts: lts.decode()
@@ -601,7 +602,7 @@ def check_ssl_ciphers(conf):
                 'specifying ssl_ciphers not supported by this installation of '
                 'Python; requires Python 2.7'
             )
-        if re.search(r'[^a-zA-z0-9, :!\-+@]', ssl_ciphers):
+        if re.search(r'[^a-zA-z0-9, :!\-+@=]', ssl_ciphers):
             raise getmailConfigurationError(
                 'invalid character in ssl_ciphers'
             )
